import { DOMConversionMap, DOMExportOutput, ElementNode, LexicalEditor, LexicalNode, NodeKey, SerializedElementNode, Spread } from 'lexical';
import { MdxJsxAttribute } from 'mdast-util-mdx-jsx';
import { MDX_NODE_TYPES, htmlTags } from './MdastHTMLNode';
export type KnownHTMLTagType = (typeof htmlTags)[number];
declare const TYPE_NAME: "generic-html";
export type SerializedGenericHTMLNode = Spread<{
    tag: KnownHTMLTagType;
    type: typeof TYPE_NAME;
    mdxType: MdxNodeType;
    attributes: MdxJsxAttribute[];
    version: 1;
}, SerializedElementNode>;
type MdxNodeType = (typeof MDX_NODE_TYPES)[number];
export declare class GenericHTMLNode extends ElementNode {
    /** @internal */
    __tag: KnownHTMLTagType;
    __nodeType: MdxNodeType;
    __attributes: MdxJsxAttribute[];
    static getType(): string;
    static clone(node: GenericHTMLNode): GenericHTMLNode;
    constructor(tag: KnownHTMLTagType, type: MdxNodeType, attributes: MdxJsxAttribute[], key?: NodeKey);
    getTag(): KnownHTMLTagType;
    getNodeType(): MdxNodeType;
    getAttributes(): MdxJsxAttribute[];
    updateAttributes(attributes: MdxJsxAttribute[]): void;
    getStyle(): string;
    createDOM(): HTMLElement;
    updateDOM(): boolean;
    static importDOM(): DOMConversionMap | null;
    exportDOM(editor: LexicalEditor): DOMExportOutput;
    static importJSON(serializedNode: SerializedGenericHTMLNode): GenericHTMLNode;
    exportJSON(): SerializedGenericHTMLNode;
    extractWithChild(): boolean;
    isInline(): boolean;
}
export declare function $createGenericHTMLNode(tag: KnownHTMLTagType, type: MdxNodeType, attributes: MdxJsxAttribute[]): GenericHTMLNode;
export declare function $isGenericHTMLNode(node: LexicalNode | null | undefined): node is GenericHTMLNode;
export {};
